@@ -41,7 +41,10 @@ Common API implementation/base classes:
     Assembly
     Projection
 
-$Id: common.py 928 2011-02-03 23:24:16Z pierre $
+:copyright: Copyright 2006-2011 by the PyNN team, see AUTHORS.
+:license: CeCILL, see LICENSE for details.
+
+$Id: common.py 994 2011-10-03 13:13:54Z apdavison $
 """
 
 import numpy, os
@@ -230,6 +233,10 @@ class IDMixin(object):
         """Set the initial value of a state variable of the cell."""
         self.parent._set_cell_initial_value(self, variable, value)
 
+    def as_view(self):
+        """Return a PopulationView containing just this cell.""" 
+        index = self.parent.id_to_index(self) 
+        return self.parent[index:index+1] 
 
 # =============================================================================
 #   Functions for simulation set-up and control
@@ -325,9 +332,9 @@ def build_connect(projection_class, connector_class):
         otherwise. Weights should be in nA or µS.
         """
         if isinstance(source, IDMixin):
-            source = source.parent
+            source = source.as_view()
         if isinstance(target, IDMixin):
-            target = target.parent
+            target = target.as_view()
         connector = connector_class(p_connect=p, weights=weight, delays=delay)
         return projection_class(source, target, connector, target=synapse_type, rng=rng)
     return connect
@@ -1350,7 +1357,20 @@ class Assembly(object):
         else:
             raise TypeError("can only add a Population or another Assembly to an Assembly")
         return self
-        
+    
+    def sample(self, n, rng=None):
+        """
+        Randomly sample n cells from the Assembly, and return a Assembly
+        object.
+        """
+        assert isinstance(n, int)
+        if not rng:
+            rng = random.NumpyRNG()
+        indices = rng.permutation(numpy.arange(len(self)))[0:n]
+        logger.debug("The %d cells recorded have indices %s" % (n, indices))
+        logger.debug("%s.sample(%s)", self.label, n)
+        return self[indices]
+    
     def initialize(self, variable, value):
         """
         Set the initial value of one of the state variables of the neurons in
@@ -1874,7 +1894,7 @@ class Projection(object):
         if max is None:
             max = weights.max()
         bins = numpy.linspace(min, max, nbins+1)
-        return numpy.histogram(weights, bins, new=True)  # returns n, bins
+        return numpy.histogram(weights, bins)  # returns n, bins
 
     def describe(self, template='projection_default.txt', engine='default'):
         """
