@@ -3,6 +3,9 @@ Defines a common implementation of the built-in PyNN Connector classes.
 
 Simulator modules may use these directly, or may implement their own versions
 for improved performance.
+
+:copyright: Copyright 2006-2011 by the PyNN team, see AUTHORS.
+:license: CeCILL, see LICENSE for details.
 """
 
 import numpy, logging, sys, re
@@ -80,7 +83,11 @@ class ConnectionAttributeGenerator(object):
             node will be smaller.
         distance_matrix - a DistanceMatrix object, used for calculating
                           distance-dependent attributes.
-        sub-mask - a list of cell ids. I don't know what this is for. Pierre?
+        sub-mask - a sublist of the ids we want compute some values with. For
+                   example in parallel, distances shoudl be computed only between a source
+                   and local targets, since only connections with those targets are established. 
+                   Avoid useless computations...                 
+                   
         """
         if isinstance(self.source, basestring):
             assert distance_matrix is not None            
@@ -105,7 +112,10 @@ class ConnectionAttributeGenerator(object):
             if sub_mask is None:
                 values = self.source.next(N, mask_local=self.local_mask)
             else:
-                values = self.source.next(N, mask_local=self.local_mask)[sub_mask]
+                data  = self.source.next(N, mask_local=self.local_mask)
+                if type(data) == numpy.float64:
+                    data = numpy.array([data])
+                values = data[sub_mask]
             return values
         elif isinstance(self.source, numpy.ndarray):
             if len(self.source.shape) == 2:
@@ -316,8 +326,11 @@ class ProbabilisticConnector(Connector):
 
         self.distance_matrix.set_source(src.position)        
         if not self.allow_self_connections and self.projection.pre == self.projection.post:
-            i         = numpy.where(self.candidates == src)[0]
-            precreate = numpy.delete(precreate, i)
+            idx_src   = numpy.where(self.candidates == src)
+            if len(idx_src) > 0:
+                i     = numpy.where(precreate == idx_src[0])
+                if len(i) > 0:
+                    precreate = numpy.delete(precreate, i[0])
                 
         if (n_connections is not None) and (len(precreate) > 0):            
             create = numpy.array([], int)
@@ -431,7 +444,7 @@ class DistanceDependentProbabilityConnector(Connector):
         `space` -- a Space object.
         `weights` -- may either be a float, a RandomDistribution object, a list/
                      1D array with at least as many items as connections to be
-                     created, or a DistanceDependence object. Units nA.
+                     created, or a distance expression as for `d_expression`. Units nA.
         `delays`  -- as `weights`. If `None`, all synaptic delays will be set
                      to the global minimum delay.
         """
@@ -762,7 +775,7 @@ class OneToOneConnector(Connector):
                         
             for tgt, src, w, d in zip(projection.post.local_cells, sources, weights, delays):
                 # the float is in case the values are of type numpy.float64, which NEST chokes on
-                projection.connection_manager.connect(src, [tgt], float(w), float(d))
+                projection.connection_manager.connect(src, [tgt], [float(w)], [float(d)])
                 self.progression(count)
                 count += 1
         else:
@@ -776,7 +789,7 @@ class SmallWorldConnector(Connector):
     parameter_names = ('allow_self_connections', 'degree', 'rewiring')
     
     def __init__(self, degree, rewiring, allow_self_connections=True,
-                 weights=0.0, delays=None, space=Space(), safe=True, verbose=False):
+                 weights=0.0, delays=None, space=Space(), safe=True, verbose=False, n_connections=None):
         """
         Create a new connector.
         
@@ -787,6 +800,7 @@ class SmallWorldConnector(Connector):
             Population to itself, this flag determines whether a neuron is
             allowed to connect to itself, or only to other neurons in the
             Population.        
+        `n_connections`  -- The number of efferent synaptic connections per neuron. 
         `weights` -- may either be a float, a RandomDistribution object, a list/
                      1D array with at least as many items as connections to be
                      created, or a DistanceDependence object. Units nA.
@@ -799,8 +813,9 @@ class SmallWorldConnector(Connector):
         self.rewiring               = rewiring    
         self.d_expression           = "d < %g" %degree        
         self.allow_self_connections = allow_self_connections
+        self.n_connections          = n_connections
         
-    def _smallworld_connect(self, src, p):
+    def _smallworld_connect(self, src, p, n_connections=None):
         """
         Connect-up a Projection with connection probability p, where p may be either
         a float 0<=p<=1, or a dict containing a float array for each pre-synaptic
@@ -810,24 +825,33 @@ class SmallWorldConnector(Connector):
         rarr = self.probas_generator.get(self.N)
         if not core.is_listlike(rarr) and numpy.isscalar(rarr): # if N=1, rarr will be a single number
             rarr = numpy.array([rarr])
-        create = numpy.where(rarr < p)[0]  
+        precreate = numpy.where(rarr < p)[0]  
         self.distance_matrix.set_source(src.position)        
         
         if not self.allow_self_connections and self.projection.pre == self.projection.post:
-            i       = numpy.where(self.candidates == src)[0]
-            create  = numpy.delete(create, i)        
+            i         = numpy.where(self.candidates == src)[0]
+            precreate = numpy.delete(precreate, i)        
         
         idx = numpy.arange(0, self.size)
         if not self.allow_self_connections and self.projection.pre == self.projection.post:
             i   = numpy.where(self.candidates == src)[0]
             idx = numpy.delete(idx, i)
         
-        rarr    = self.probas_generator.get(self.N)[create]
+        rarr    = self.probas_generator.get(self.N)[precreate]
         rewired = numpy.where(rarr < self.rewiring)[0]
         N       = len(rewired)
         if N > 0:
-            new_idx         = (len(idx)-1) * self.probas_generator.get(self.N)[create]
-            create[rewired] = idx[new_idx.astype(int)]
+            new_idx            = (len(idx)-1) * self.probas_generator.get(self.N)[precreate]
+            precreate[rewired] = idx[new_idx.astype(int)]
+        
+        if (n_connections is not None) and (len(precreate) > 0):            
+            create = numpy.array([], int)
+            while len(create) < n_connections: # if the number of requested cells is larger than the size of the
+                                               ## presynaptic population, we allow multiple connections for a given cell
+                create = numpy.concatenate((create, self.projection.rng.permutation(precreate)))
+            create = create[:n_connections]
+        else:
+            create = precreate 
 
         targets = self.candidates[create]
         weights = self.weights_generator.get(self.N, self.distance_matrix, create)
@@ -856,7 +880,7 @@ class SmallWorldConnector(Connector):
         for count, src in enumerate(projection.pre.all()):     
             self.distance_matrix.set_source(src.position)
             proba = proba_generator.get(self.N, self.distance_matrix).astype(float)
-            self._smallworld_connect(src, proba)
+            self._smallworld_connect(src, proba, self.n_connections)
             self.progression(count)
 
 
@@ -870,7 +894,7 @@ class CSAConnector(Connector):
             min_delay = common.get_min_delay()
             Connector.__init__(self, None, None, safe=safe, verbose=verbose)
             self.cset = cset
-            if cset.arity == 0:
+            if csa.arity(cset) == 0:
                 #assert weights is not None and delays is not None, \
                 #       'must specify weights and delays in addition to a CSA mask'
                 self.weights = weights
@@ -880,7 +904,7 @@ class CSAConnector(Connector):
                 if delays is None:
                     self.delays = common.get_min_delay()
             else:
-                assert cset.arity == 2, 'must specify mask or connection-set with arity 2'
+                assert csa.arity(cset) == 2, 'must specify mask or connection-set with arity 2'
                 assert weights is None and delays is None, \
                        "weights or delays specified both in connection-set and as CSAConnector argument"
     else:
@@ -898,23 +922,18 @@ class CSAConnector(Connector):
 
     def connect(self, projection):
         """Connect-up a Projection."""
-        i0 = projection.pre.first_id
-        size1 = projection.pre.last_id - i0
-        j0 = projection.post.first_id
-        targets = [j - j0 for j in projection.post]
-
-        # Cut out finite part and shift to global ids
-        c = csa.shift (i0, j0) * csa.cross ((0, size1), targets) * self.cset
+        # Cut out finite part
+        c = csa.cross((0, projection.pre.size-1), (0, projection.post.size-1)) * self.cset
         
-        if csa.arity (self.cset) == 2:
+        if csa.arity(self.cset) == 2:
             # Connection-set with arity 2
             for (i, j, weight, delay) in c:
-                projection.connection_manager.connect (i, [j], weight, delay)
+                projection.connection_manager.connect (projection.pre[i], [projection.post[j]], weight, delay)
         elif CSAConnector.isConstant (self.weights) \
              and CSAConnector.isConstant (self.delays):
             # Mask with constant weights and delays
             for (i, j) in c:
-                projection.connection_manager.connect (i, [j], self.weights, self.delays)
+                projection.connection_manager.connect (projection.pre[i], [projection.post[j]], self.weights, self.delays)
         else:
             # Mask with weights and/or delays iterable
             weights = self.weights
@@ -924,4 +943,4 @@ class CSAConnector(Connector):
             if CSAConnector.isConstant (delays):
                 delays = CSAConnector.constantIterator (delays)
             for (i, j), weight, delay in zip (c, weights, delays):
-                projection.connection_manager.connect (i, [j], weight, delay)
+                projection.connection_manager.connect (projection.pre[i], [projection.post[j]], weight, delay)
