@@ -10,7 +10,7 @@ Classes:
 :copyright: Copyright 2006-2011 by the PyNN team, see AUTHORS.
 :license: CeCILL, see LICENSE for details.
 
-$Id: electrodes.py 957 2011-05-03 13:44:15Z apdavison $
+$Id: electrodes.py 1121 2012-04-06 13:58:02Z apdavison $
 """
 
 import nest
@@ -23,7 +23,11 @@ from pyNN.common import Population, PopulationView, Assembly
 
 class CurrentSource(object):
     """Base class for a source of current to be injected into a neuron."""
-    
+
+    def delay_correction(self, value):
+        # use dt or min_delay?
+        return value - state.min_delay
+
     def inject_into(self, cell_list):
         """Inject this current source into some cells."""
         for id in cell_list:
@@ -31,34 +35,36 @@ class CurrentSource(object):
                 raise TypeError("Can't inject current into a spike source.")
         if isinstance(cell_list, (Population, PopulationView, Assembly)):
             cell_list = [cell for cell in cell_list]
-        nest.DivergentConnect(self._device, cell_list)
-    
+        nest.DivergentConnect(self._device, cell_list, delay=state.min_delay, weight=1.0)
+
 
 class DCSource(CurrentSource):
     """Source producing a single pulse of current of constant amplitude."""
-    
+
     def __init__(self, amplitude=1.0, start=0.0, stop=None):
         """Construct the current source.
-        
+
         Arguments:
             start     -- onset time of pulse in ms
             stop      -- end of pulse in ms
             amplitude -- pulse amplitude in nA
         """
+        self.start = start
+        self.stop = stop
         self.amplitude = amplitude
         self._device = nest.Create('dc_generator')
         nest.SetStatus(self._device, {'amplitude': 1000.0*self.amplitude,
-                                      'start': float(start)}) # conversion from nA to pA
+                                      'start': self.delay_correction(start)}) # conversion from nA to pA
         if stop:
-            nest.SetStatus(self._device, {'stop': float(stop)})
+            nest.SetStatus(self._device, {'stop': self.delay_correction(stop)})
 
 
 class ACSource(CurrentSource):
     """Source producing a single pulse of current of constant amplitude."""
-    
+
     def __init__(self, amplitude=1.0, offset=0.0, frequency=10, phase=0., start=0.0, stop=None):
         """Construct the current source.
-        
+
         Arguments:
             start     -- onset time of pulse in ms
             stop      -- end of pulse in ms
@@ -77,23 +83,23 @@ class ACSource(CurrentSource):
                                       'offset'   : 1000.0*self.offset,
                                       'frequency': float(self.frequency),
                                       'phase'    : float(self.phase),
-                                      'start'    : float(start)}) # conversion from nA to pA
+                                      'start'    : self.delay_correction(start)}) # conversion from nA to pA
         if stop:
             nest.SetStatus(self._device, {'stop': float(stop)})
 
-        
-        
+
+
 class StepCurrentSource(CurrentSource):
     """A step-wise time-varying current source."""
-    
+
     def __init__(self, times, amplitudes):
         """Construct the current source.
-        
+
         Arguments:
             times      -- list/array of times at which the injected current changes.
             amplitudes -- list/array of current amplitudes to be injected at the
                           times specified in `times`.
-                          
+
         The injected current will be zero up until the first time in `times`. The
         current will continue at the final value in `amplitudes` until the end
         of the simulation.
@@ -108,10 +114,10 @@ class StepCurrentSource(CurrentSource):
             amplitudes.append(amplitudes[-1])  # bug in NEST
         except AttributeError:
             numpy.append(amplitudes, amplitudes[-1])
-        nest.SetStatus(self._device, {'amplitude_times': numpy.array(times, 'float'),
+        nest.SetStatus(self._device, {'amplitude_times': self.delay_correction(numpy.array(times, 'float')),
                                       'amplitude_values': 1000.0*numpy.array(amplitudes, 'float')})
-        
-        
+
+
 class NoisyCurrentSource(CurrentSource):
     """A Gaussian "white" noise current source. The current amplitude changes at fixed
     intervals, with the new value drawn from a Gaussian distribution."""
@@ -123,14 +129,14 @@ class NoisyCurrentSource(CurrentSource):
     # simulator-independent noise, which means adding an rng argument. If this
     # is a NativeRNG, we use the 'noise_generator' model, otherwise we generate
     # values and use a 'step_current_generator'.
-    
+
     def __init__(self, mean, stdev, dt=None, start=0.0, stop=None, rng=None):
         """Construct the current source.
-        
+
         Required arguments:
             mean  -- mean current amplitude in nA
             stdev -- standard deviation of the current amplitude in nA
-            
+
         Optional arguments:
             dt    -- interval between updates of the current amplitude. Must be
                      a multiple of the simulation time step. If not specified,
@@ -157,9 +163,9 @@ class NoisyCurrentSource(CurrentSource):
             self._device = nest.Create('noise_generator')
             nest.SetStatus(self._device, {'mean': mean*1000.0,
                                            'std': stdev*1000.0,
-                                           'start': float(start),
+                                           'start': self.delay_correction(start),
                                            'dt': self.dt})
             if stop:
-                nest.SetStatus(self._device, {'stop': float(stop)})
+                nest.SetStatus(self._device, {'stop': self.delay_correction(stop)})
         else:
             raise NotImplementedError("Only using a NativeRNG is currently supported.")
