@@ -14,7 +14,7 @@ Classes:
 :copyright: Copyright 2006-2011 by the PyNN team, see AUTHORS.
 :license: CeCILL, see LICENSE for details.
 
-$Id: files.py 957 2011-05-03 13:44:15Z apdavison $
+$Id: files.py 1194 2012-07-03 15:00:07Z apdavison $
 """
 
 
@@ -68,7 +68,7 @@ def savez(file, *args, **kwds):
     for key, val in namedict.iteritems():
         fname = key + '.npy'
         filename = os.path.join(direc, fname)
-        fid = open(filename,'wb')
+        fid = open(filename, 'wb')
         format.write_array(fid, numpy.asanyarray(val))
         fid.close()
         zip.write(filename, arcname=fname)
@@ -191,17 +191,22 @@ class NumpyBinaryFile(BaseFile):
     arrays.
     """
     
+    def _check_open(self):
+        if not hasattr(self, "fileobj") or self.fileobj.closed:
+            self.fileobj = open(self.name, self.mode, DEFAULT_BUFFER_SIZE)
+        else:
+            self.fileobj.seek(0)
+    
     def write(self, data, metadata):
         __doc__ = BaseFile.write.__doc__
         self._check_open()
-        metadata_array = numpy.array(metadata.items())
+        metadata_array = numpy.array(metadata.items(), dtype=(str, float))
         savez(self.fileobj, data=data, metadata=metadata_array)
         
     def read(self):
         __doc__ = BaseFile.read.__doc__
         self._check_open()
         data = numpy.load(self.fileobj)['data']
-        self.fileobj.seek(0)
         return data
     
     def get_metadata(self):
@@ -213,7 +218,6 @@ class NumpyBinaryFile(BaseFile):
                 D[name] = eval(value)
             except Exception:
                 D[name] = value
-        self.fileobj.seek(0)
         return D
     
     
